/************************************************************************/
/* Licensed Materials – Property of  HCL                                */
/* (c) Copyright HCL Technologies Ltd. 2016, 2017 All rights reserved.  */
/* Trademark of HCL Technologies Limited                                */
/************************************************************************/

const fs = require('fs-extra');
const async = require('async');
var workManager = require('../modules/workManager');

module.exports = function () {
	var Importer = {};

	Importer.doImport = function (expImpPath, deleteAllWhiteboards) {
		var config = require('../modules/config');
		var Rooms = require('../models/rooms');

		var roomWhiteboards = [];
		var roomFileWhiteboards = [];
		var roomWhiteboardsUnaltered = [];
		async.waterfall([
			// find Room in DB
			function (callback) {
				workManager.workStarted();
				Rooms.findOrCreate({
					"_id": "root",
					"name": "_root"
				}, function (err, room) {
					callback(err, room);	// note rooms[0] will be room arg for next function
				});
			},
			function (room, callback) {
				// read db whiteboards into map for quick access
				roomWhiteboards = [];
				roomWhiteboardsUnaltered=[]
				room.whiteboards.forEach(function (wb) {
					roomWhiteboards[wb.graph_id] = wb;
					if(!deleteAllWhiteboards){
						roomWhiteboardsUnaltered[wb.graph_id] = wb;
					}
				});

				fs.ensureDir(expImpPath, function (err) {
					callback(err, room);
				});
			},
			function (room, callback) {
				var roomFile = expImpPath + '/' + 'rooms.json';
				fs.readJson(roomFile, { spaces: 4 }, function (err, fileRoom) {
					callback(err, room, fileRoom);
				});
			},
			function (room, fileRoom, callback){
				if(deleteAllWhiteboards){
					//the whiteboard delections tasks
					var asyncTasks = [];

					// remove all whiteboards no
					room.whiteboards.forEach(function (wb) {
						asyncTasks.push(asyncFunctionBuilder(wb));
					});
					//winston.info(asyncTasks);
					async.parallel(asyncTasks,
						// optional callback
						function(err, results) {
							if(err){
								callback(err);
							}
							//log result 
							winston.info("Async Parallel done", results);
							room.set("whiteboards",[]);
							roomWhiteboards=[];
							room.save(function (err) {
								callback(err, room, fileRoom);
							});	
						});
				}else{
					callback(null, room, fileRoom)
				}
			},
			function (room, fileRoom, callback) {
				if(!fileRoom["references"]){
					winston.info("Importing files from previous data format");
				}
				//IMPORTANT: FOR DRLIVE SERVER SOCKET EMISSION EVENTS
				winston.info("Importing Wbs Count :"+fileRoom.whiteboards.length)
				// read into map for quick access
				roomFileWhiteboards = [];
				fileRoom.whiteboards.forEach(function (wb) {
					roomFileWhiteboards[wb.graph_id] = wb;
				});

				fileRoom.whiteboards.forEach(function (wb) {
					var graph_id = wb.graph_id;
					if (roomWhiteboards[graph_id] != null) {
						// existing graph
						// update
						roomWhiteboards[graph_id] = wb;
				//		winston.info('Importing Whiteboard - ', wb.wname, ' : ', wb.graph_id);
					//	importWhiteboard(expImpPath, graph_id);
					}
					else {//addition of new whiteboard(from file) not currently present in db
						// new graph
						room.whiteboards.push(wb);
						roomWhiteboards[graph_id] = wb;
					//	winston.info('Importing Whiteboard (new) - ', wb.wname, ' : ', wb.graph_id);
					//	importWhiteboard(expImpPath, graph_id);
					}

				});
				var i = 0;
				var import_executed=false;
				var ifunc = setInterval(function(){
					if(i<fileRoom.whiteboards.length){
						if(!import_executed){
							import_executed=true;
							importWhiteboard(expImpPath,room._id,fileRoom.whiteboards[i].graph_id, function(err){
								if(err) callback(err)
								i++;
								import_executed=false;
							});
						}
	
					}
					if(i==fileRoom.whiteboards.length){
						clearInterval(ifunc);
						if(fileRoom["references"]){
							room.set("references",fileRoom["references"]);
						}
						room.save(function (err) {
							callback(err);
						});						
					}

				},100);
			},
			function(callback) {
				importModelElements(expImpPath, roomFileWhiteboards,roomWhiteboardsUnaltered,deleteAllWhiteboards,function(err){
					requester.resolveAllModelChanges(function(result){
						workManager.workEnded();//Import model elements
						//winston.info(workManager.getWorkCount());
						callback(result);
					})
				});
			}
		],
			function (err, result) {
				// all done
				if (Rooms != null) {
					Rooms.closeConnection();
				}
				if(err)
				{
					winston.error(err);
					process.exit(1);
				}

				workManager.workEnded();
			});

	};

	function asyncFunctionBuilder(wb){
		var func = function(callback){
			var graph_id = wb.graph_id;
			// delete db for this wb
			winston.info('Deleting Whiteboard - ', wb.wname, ' : ', wb.graph_id);
			var DB = require('../models/db')(graph_id);
			DB.remove({}, function(err) { 
				//console.log('collection removed')
				if(err)
					callback(err);
				DB.closeConnection();
				callback(null, graph_id);							
			});			
		}
		return func;
	}
	function importWhiteboard(expImpPath, room_id, graph_id, done) {
		var dbFile = expImpPath + '/' + graph_id + '.json';
		var DB = null;
		winston.info('Importing Whiteboard - '+graph_id);

		async.waterfall([
			// read cells from file
			function (callback) {
				workManager.workStarted();
				fs.readJson(dbFile, { spaces: 4 }, function (err, cellsFile) {
					callback(err, cellsFile);
				});
			},
			// empty out existing cells in DB
			function (cellsFile, callback) {
				requester.clearWhiteboard(room_id, graph_id, function(err){
					callback(err, cellsFile);
				});
			},
			// insert the cells read from file
			function (cellsFile, callback) {
				
				DB = require('../models/db')(graph_id);
				
				// update the _id so that Mongo does not generate its own
				cellsFile.forEach(function (cell) {
					cell._id = cell.id;
				});
				var firstRecord= cellsFile[0];
				if(firstRecord){
					DB.create(firstRecord, function(err, doc) {
						if(!err){
							cellsFile.shift();
							DB.insertMany(cellsFile, function (err) {
								if(err)
									callback(err);
								callback(null);
							});
						}else{
							callback(err);
						}
					});
				}
				else{
					callback(null);
				}
				/*requester.addCells(room_id, graph_id, cellsFile, function(result){

						callback();
				});*/
			}
		],
			function (err, result) {
				// all done
				if (err) {
					winston.info('Import Whiteboard failed - ', graph_id, ' Error - ', err);
				}
				if (DB != null) {
					DB.closeConnection();
				}
				workManager.workEnded();
				done(err);
			});
	}

	function importModelElements(expImpPath, wbToImportList, roomDBWbList,deleteAllModelElements, done) {
		winston.info('Importing Model Elements...');
		var meFile = expImpPath + '/' + 'modelElements.json';
		var DB = null;

		async.waterfall([
			// read cells from file
			function (callback) {
				workManager.workStarted();
				fs.readJson(meFile, { spaces: 4 }, function (err, cellsFile) {
					callback(err, cellsFile);
				});
			},
			//Remove only the elements that will be imported
			function (cellsFile, callback) {
				var ids_to_insert = cellsFile.map(function(o){return o._id;})//get ids of elements about to be inserted
				DB = require('../models/modelElement');
				if(!deleteAllModelElements){
					DB.find( { _id: { $in: ids_to_insert } }, function (err, db_records) {//get all db elements about to be replaced
						DB.remove( { _id: { $in: ids_to_insert } }, function (err) {//Remove only the elements about to be inserted
							callback(err, updateModelsReferences(cellsFile,db_records, wbToImportList, roomDBWbList));
						});
					});
				}
				else{
					DB.remove({}, function (err) {//Remove all model elements
						callback(err, updateModelsReferences(cellsFile, null, wbToImportList, roomDBWbList));
					});					
				}
			},
			// insert the cells read from file
			function (cellsFile, callback) {
				DB.insertMany(cellsFile, function (err) {
					callback(err, cellsFile);
					
				});
			}
		],
			function (err, cellsFile) {
				// all done
				if (err) {
					winston.info('Import Model Elements failed - ', ' Error - ', err);
					done(err);
				}
				else {
					winston.info(cellsFile.length + ' elements inserted.');
				}

				if (DB != null) {
					DB.closeConnection();
				}

				done(null);
			});
	}
	//For all model elements about to be inserted in modelelements collection, merge the file model element reference with the db model element references so no instantiation is left out
	function updateModelsReferences(fileModels, dbModels, lsImportWbsFile, dbWbsList){
		if(!dbModels){//when all models are deleted as part of import
			dbModels="[]";
		}
		dbModels = JSON.parse(JSON.stringify(dbModels));
		for(var x=0;x<fileModels.length;x++){
			var found= false;
			var fileEl= fileModels[x];
			for(var y=0; y<dbModels.length;y++){
				var dbEl = dbModels[y];
				if(fileEl._id == dbEl._id && JSON.stringify(fileEl.references)!=JSON.stringify(dbEl.references)){//only update if db model element and file model elements references are different
					found= true;
					var db_ref_ids = (dbEl.references)?Object.keys(dbEl.references):[];
					//append to file model all new/missing graph ids from the db
					for(var i=0; i<db_ref_ids.length; i++){
						if(!fileEl.references || !fileEl.references.hasOwnProperty(db_ref_ids[i])){//whiteboard reference is different
							if(!fileEl.references) fileModels[x]["references"]={} //initialize
							fileModels[x].references[db_ref_ids[i]]=dbEl.references[db_ref_ids[i]];
						}
						else if(fileEl.references.hasOwnProperty(db_ref_ids[i])){
							//Compare array of elements in individual whiteboards references which can be different if new element was added to db(and whiteboard is not being imported)
							if(JSON.stringify(fileEl.references[db_ref_ids[i]])!=JSON.stringify(dbEl.references[db_ref_ids[i]])){
								var wbFileElIds = fileEl.references[db_ref_ids[i]];//get ids of all elements in file
								for(var j=0; j<dbEl.references[db_ref_ids[i]].length;j++){
										if(!wbFileElIds.includes(dbEl.references[db_ref_ids[i]][j]._id)){
											fileModels[x].references[db_ref_ids[i]].push(dbEl.references[db_ref_ids[i]][j]);
									}
								}
							}
						}					
					}		
					//Remove deleted references not being exported and no longer in db(deleted after last export)
					var file_ref_ids = (fileEl.references)?Object.keys(fileEl.references):[];
					for(var i=0; i<file_ref_ids.length; i++){
						if(!lsImportWbsFile[file_ref_ids[i]]&&(!dbEl.references||!dbEl.references.hasOwnProperty(file_ref_ids[i]))){
							delete fileModels[x].references[file_ref_ids[i]];
						}					
					}									
				}
			}
			//Remove orphan references(reference not in exported file nor db)
			var file_ref_ids = (fileEl.references)?Object.keys(fileEl.references):[];
			for(var i=0; i<file_ref_ids.length; i++){
				if(!lsImportWbsFile[file_ref_ids[i]] && !dbWbsList[file_ref_ids[i]]){
					delete fileModels[x].references[file_ref_ids[i]];
				}
				if(found==false){//db does not have model element
					//remove reference to elements deleted from a whiteboard after export and wb still present in db
					if((!lsImportWbsFile[file_ref_ids[i]])){
						delete fileModels[x].references[file_ref_ids[i]];						
					}

				}					
			}	
		}

		/*
		for(var x=0;x<fileModels.length;x++){
			var fileEl= fileModels[x];
			for(var y=0; y<dbModels.length;y++){
				var dbEl = dbModels[y];
				if(fileEl._id == dbEl._id && JSON.stringify(fileEl.references)!=JSON.stringify(dbEl.references)){//only update if db model element and file model elements references are different
					var db_ref_ids = (dbEl.references)?Object.keys(dbEl.references):[];
					//append to file model all new/missing graph ids from the db
					for(var i=0; i<db_ref_ids.length; i++){
						if(!fileEl.references.hasOwnProperty(db_ref_ids[i])){//whiteboard reference is different
							fileModels[x].references[db_ref_ids[i]]=dbEl.references[db_ref_ids[i]];
						}
						else{//whiteboard reference is the same does not matter since whiteboard is overriden
							//Key(whiteboard id) in references
							var keyWhiteboard = db_ref_ids[i];
							//Compare array of elements in individual whiteboards references which can be different if new element was added to db
							if(JSON.stringify(fileEl.references[keyWhiteboard])!=JSON.stringify(dbEl.references[keyWhiteboard])){
								var wbFileElIds = fileEl.references[keyWhiteboard].map(function(o){return o._id;})//get ids of all elements in file
								for(var j=0; j<dbEl.references[keyWhiteboard].length;j++){
										if(!wbFileElIds.includes(dbEl.references[keyWhiteboard][j]._id)){
											fileEl.references[keyWhiteboard].push(dbEl.references[keyWhiteboard][j]);
									}
								}

							}

						}
					}
				}
				break;
			}
			/*Check if references need to be removed
			var file_el_ref_ids = Object.keys(fileModels[x].references);	
			//Remove graph ids no longer existing in dbs graph ids from the file
			for(var j=0; j<file_el_ref_ids.length; j++){
					//if graph reference is not in db and is not being imported then remove
					if(!lsImportWbsFile[file_el_ref_ids[j]]){//whiteboard reference is different
						delete fileModels[x].references[file_el_ref_ids[j]];
					}
			}*/				
	/*	
		}
*/
		//return updated file models
		return fileModels;

	}

	return Importer;
};