/************************************************************************/
/* Licensed Materials – Property of  HCL                                */
/* (c) Copyright HCL Technologies Ltd. 2018 All rights reserved.  */
/* Trademark of HCL Technologies Limited                                */
/************************************************************************/

const fs = require('fs-extra');
var archiver = require("archiver");
var workManager = require('../modules/workManager');

var Zipper = { "folder_path": null };
Zipper.init = function (data_file_folder) {
	Zipper.data_folder_path = data_file_folder;
	Zipper.createZipFolder();

}
//Ensures zips folder is present
Zipper.createZipFolder = function () {
	if (!fs.existsSync(Zipper.data_folder_path + "/../../zips/")) {
		fs.mkdirSync(Zipper.data_folder_path + "/../../zips/");
	}
}
Zipper.doZip = function (done) {
	// create a file to stream archive data to.
	var output = fs.createWriteStream(Zipper.data_folder_path + "/../../zips/" + Zipper.data_folder_path.substring(Zipper.data_folder_path.lastIndexOf('/')) + '.zip');
	var archive = archiver('zip', {
		zlib: { level: 9 } // Sets the compression level.
	});

	// listen for all archive data to be written
	// 'close' event is fired only when a file descriptor is involved
	output.on('close', function () {
		console.log(archive.pointer() + ' total bytes');
		console.log('archiver has been finalized and the output file descriptor has closed.');
		done(null);
	});

	// This event is fired when the data source is drained no matter what was the data source.
	// It is not part of this library but rather from the NodeJS Stream API.
	// @see: https://nodejs.org/api/stream.html#stream_event_end
	output.on('end', function () {
		console.log('Data has been drained');
	});

	// good practice to catch warnings (ie stat failures and other non-blocking errors)
	archive.on('warning', function (err) {
		if (err.code === 'ENOENT') {
			// log warning
		} else {
			// throw error
			done(err);
		}
	});

	// good practice to catch this error explicitly
	archive.on('error', function (err) {
		done(err);
	});

	// pipe archive data to the file
	archive.pipe(output);

	// append files from a sub-directory and naming it `new-subdir` within the archive
	archive.directory(Zipper.data_folder_path, Zipper.data_folder_path.substring(Zipper.data_folder_path.lastIndexOf('/')));

	// finalize the archive (ie we are done appending files but streams have to finish yet)
	// 'close', 'end' or 'finish' may be fired right after calling this method so register to them beforehand
	archive.finalize();


}

module.exports = Zipper;